@@ -12,5 +12,5 @@ export ANDROID_TC="${ANDROID_TC:-$HOME/android-tc}"
 export ANDROID_TC_ARM="${ANDROID_TC_ARM:-$ANDROID_TC/arm}"
 
 if [ ! -d "$ANDROID_TC_ARM/arm-linux-androideabi" ]; then
-    "$NDK_ROOT/build/tools/make-standalone-toolchain.sh" --platform=android-9 --arch=arm --install-dir="$ANDROID_TC_ARM" "$@"
+    "$NDK_ROOT/build/tools/make-standalone-toolchain.sh" --verbose --platform=android-19 --arch=arm --stl=libc++ --install-dir="$ANDROID_TC_ARM" "$@"
 fi
