@@ -7,7 +7,8 @@ set -e
 
 # Script to compile OpenSSL for Android
 
-COMPILEOPENSSL="yes"
+COMPILEOPENSSL="${COMPILEOPENSSL:-yes}"
+STDOUT_TARGET="${STDOUT_TARGET:-/dev/stdout}"
 CONFIGURE="yes"
 MAKECLEAN="yes"
 
@@ -32,20 +33,20 @@ export GDB_CFLAGS="--sysroot=$TCSYSROOT -Wall -g -I$TCINCLUDES/include"
 # Prepare android toolchain and environment
 ./build_androidtc_arm.sh
 
-if [ -n "$COMPILEOPENSSL" ]; then
-echo "================building openssl from $OPENSSL============================="
-cd "$OPENSSL"
-if [ -n "$MAKECLEAN" ]; then
-make clean
-fi
-if [ -n "$CONFIGURE" ]; then
-./Configure linux-generic32 no-shared no-dso -DL_ENDIAN --openssldir="$TCINCLUDES/ssl"
-#override flags in Makefile
-sed -e "s/^CFLAG=.*$/`grep -e \^CFLAG= Makefile` \$(CFLAGS)/g
+if [ "$COMPILEOPENSSL" = "yes" ]; then
+    echo "===== building openssl for arm from $OPENSSL ====="
+    cd "$OPENSSL"
+    if [ -n "$MAKECLEAN" ]; then
+        make clean 1>$STDOUT_TARGET 2>&1
+    fi
+    if [ -n "$CONFIGURE" ]; then
+        ./Configure linux-generic32 no-shared no-dso -DL_ENDIAN --openssldir="$TCINCLUDES/ssl" 1>$STDOUT_TARGET
+        #override flags in Makefile
+        sed -e "s/^CFLAG=.*$/`grep -e \^CFLAG= Makefile` \$(CFLAGS)/g
 s%^INSTALLTOP=.*%INSTALLTOP=$TCINCLUDES%g" Makefile > Makefile.out
-mv Makefile.out Makefile
-fi
-make
-make install_sw
-echo "========================openssl DONE=================================="
+        mv Makefile.out Makefile
+    fi
+    make 1>$STDOUT_TARGET
+    make install_sw 1>$STDOUT_TARGET
+    echo "===== openssl for arm build done ====="
 fi
