@@ -273,6 +273,9 @@ void ACTIVE_TASK::cleanup_task() {
     kill_subsidiary_processes();
 
     if (cc_config.exit_after_finish) {
+        msg_printf(wup->project, MSG_INFO,
+            "exit_after_finish: job %s, slot %d", wup->name, slot
+        );
         gstate.write_state_file();
         exit(0);
     }
@@ -283,6 +286,13 @@ int ACTIVE_TASK::init(RESULT* rp) {
     wup = rp->wup;
     app_version = rp->avp;
     max_elapsed_time = rp->wup->rsc_fpops_bound/rp->avp->flops;
+    if (max_elapsed_time < MIN_TIME_BOUND) {
+        msg_printf(wup->project, MSG_INFO,
+            "Elapsed time limit %f < %f; setting to %f",
+            max_elapsed_time, MIN_TIME_BOUND, DEFAULT_TIME_BOUND
+        );
+        max_elapsed_time = DEFAULT_TIME_BOUND;
+    }
     max_disk_usage = rp->wup->rsc_disk_bound;
     max_mem_usage = rp->wup->rsc_memory_bound;
     get_slot_dir(slot, slot_dir, sizeof(slot_dir));
@@ -1072,7 +1082,7 @@ int ACTIVE_TASK::handle_upload_files() {
                     "Can't find uploadable file %s", p
                 );
             }
-            snprintf(path, sizeof(path), "%s/%s", slot_dir, buf);
+            snprintf(path, sizeof(path), "%.*s/%.*s", DIR_LEN, slot_dir, FILE_LEN, buf);
             delete_project_owned_file(path, true);  // delete the link file
         }
     }
