@@ -131,14 +131,22 @@ bool ACTIVE_TASK_SET::poll() {
     static double last_finish_check_time = 0;
     if (gstate.clock_change || gstate.now - last_finish_check_time > 10) {
         last_finish_check_time = gstate.now;
+        int exit_code;
         for (i=0; i<active_tasks.size(); i++) {
             ACTIVE_TASK* atp = active_tasks[i];
             if (atp->task_state() == PROCESS_UNINITIALIZED) continue;
             if (atp->finish_file_time) {
-                // process is still there 10 sec after it wrote finish file.
-                // abort the job
-                atp->abort_task(EXIT_ABORTED_BY_CLIENT, "finish file present too long");
-            } else if (atp->finish_file_present()) {
+                if (gstate.now - atp->finish_file_time > FINISH_FILE_TIMEOUT) {
+                    // process is still there 5 min after it wrote finish file.
+                    // abort the job
+                    // Note: actually we should treat it as successful.
+                    // But this would be tricky.
+                    //
+                    atp->abort_task(EXIT_ABORTED_BY_CLIENT,
+                        "Process still present 5 min after writing finish file; aborting"
+                    );
+                }
+            } else if (atp->finish_file_present(exit_code)) {
                 atp->finish_file_time = gstate.now;
             }
         }
@@ -463,7 +471,8 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             // if another process killed the app, it looks like exit(0).
             // So check for the finish file
             //
-            if (finish_file_present()) {
+            int e;
+            if (finish_file_present(e)) {
                 set_task_state(PROCESS_EXITED, "handle_exited_app");
                 break;
             }
@@ -518,6 +527,7 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             double x;
             char buf[256];
             bool is_notice;
+            int e;
             if (temporary_exit_file_present(x, buf, is_notice)) {
                 handle_temporary_exit(will_restart, x, buf, is_notice);
             } else {
@@ -532,11 +542,11 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
                     snprintf(err_msg, sizeof(err_msg),
                         "process exited with code %d (0x%x, %d)",
                         result->exit_status, result->exit_status,
-                        (-1<<8)|result->exit_status
+                        (~0xff)|result->exit_status
                     );
                     gstate.report_result_error(*result, err_msg);
                 } else {
-                    if (finish_file_present()) {
+                    if (finish_file_present(e)) {
                         set_task_state(PROCESS_EXITED, "handle_exited_app");
                     } else {
                         handle_premature_exit(will_restart);
@@ -611,19 +621,28 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 
 // structure of a finish file (see boinc_api.cpp)):
 // exit status (int)
-// message
-// "notice" or blank line
-// ... or empty
+// optional:
+//  message to show user
+//  "notice" or blank line
 //
-bool ACTIVE_TASK::finish_file_present() {
+bool ACTIVE_TASK::finish_file_present(int &exit_code) {
     char path[MAXPATHLEN], buf[1024], buf2[256];
     safe_strcpy(buf, "");
     safe_strcpy(buf2, "");
+
+    exit_code = 0;
+
     sprintf(path, "%s/%s", slot_dir, BOINC_FINISH_CALLED_FILE);
     FILE* f = boinc_fopen(path, "r");
     if (!f) return false;
-    fgets(buf, sizeof(buf), f);     // read (and discard) exit status
     char* p = fgets(buf, sizeof(buf), f);
+    if (p && strlen(buf)) {
+        int e;
+        if (sscanf(buf, "%d", &e) == 1) {
+            exit_code = e;
+        }
+    }
+    p = fgets(buf, sizeof(buf), f);
     if (p && strlen(buf)) {
         fgets(buf2, sizeof(buf2), f);
         msg_printf(result->project,
@@ -1535,7 +1554,7 @@ void ACTIVE_TASK_SET::get_msgs() {
         }
         if (atp->get_app_status_msg()) {
             if (old_time != atp->checkpoint_cpu_time) {
-                char buf[256];
+                char buf[512];
                 sprintf(buf, "%s checkpointed", atp->result->name);
                 if (atp->overdue_checkpoint) {
                     gstate.request_schedule_cpus(buf);
@@ -1550,11 +1569,6 @@ void ACTIVE_TASK_SET::get_msgs() {
                         "[checkpoint] result %s checkpointed",
                         atp->result->name
                     );
-                } else if (log_flags.task_debug) {
-                    msg_printf(atp->wup->project, MSG_INFO,
-                        "[task] result %s checkpointed",
-                        atp->result->name
-                    );
                 }
                 atp->write_task_state_file();
             }
