@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2010 University of California
+// Copyright (C) 2018 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -20,6 +20,8 @@
 
 #include "gui_http.h"
 
+#define DEFAULT_VERSION_CHECK_URL "https://boinc.berkeley.edu/download.php?xml=1"
+
 struct GET_CURRENT_VERSION_OP: public GUI_HTTP_OP {
     int error_num;
 
@@ -34,4 +36,19 @@ struct GET_CURRENT_VERSION_OP: public GUI_HTTP_OP {
 
 extern void newer_version_startup_check();
 
+struct NVC_CONFIG {
+    std::string client_download_url;
+    std::string client_new_version_name;
+    std::string client_version_check_url;
+    std::string network_test_url;
+
+    NVC_CONFIG();
+    void defaults();
+    int parse(FILE*);
+};
+
+extern NVC_CONFIG nvc_config;
+
+extern int read_vc_config_file(void);
+
 #endif
