@@ -145,7 +145,7 @@ static bool compare_pci_slots(int NVIDIA_GPU_Index1, int NVIDIA_GPU_Index2) {
 // -1 if the OS version we are running on is less than x.y
 //  0 if the OS version we are running on is equal to x.y
 // +1 if the OS version we are running on is lgreater than x.y
-static int compareOSVersionTo(int toMajor, int toMinor) {
+int compareOSVersionTo(int toMajor, int toMinor) {
     static SInt32 major = -1;
     static SInt32 minor = -1;
 
@@ -202,6 +202,7 @@ void COPROCS::get_opencl(
     vector<int>devnums_pci_slot_sort;
     vector<OPENCL_DEVICE_PROP>::iterator it;
     int max_other_coprocs = MAX_RSC-1;  // coprocs[0] is reserved for CPU
+    string s;
 
     if (cc_config.no_opencl) {
         return;
@@ -319,7 +320,7 @@ void COPROCS::get_opencl(
         }
 
         for (device_index=0; device_index<num_devices; ++device_index) {
-            memset(&prop, 0, sizeof(prop));
+            prop.clear();
             prop.device_id = devices[device_index];
             strlcpy(
                 prop.opencl_platform_version, platform_version,
@@ -405,7 +406,7 @@ void COPROCS::get_opencl(
         }
 
         for (device_index=0; device_index<num_devices; ++device_index) {
-            memset(&prop, 0, sizeof(prop));
+            prop.clear();
             prop.device_id = devices[device_index];
             strlcpy(
                 prop.opencl_platform_version, platform_version,
@@ -486,6 +487,9 @@ void COPROCS::get_opencl(
                     COPROC_NVIDIA c;
                     c.opencl_prop = prop;
                     c.set_peak_flops();
+                    if (c.bad_gpu_peak_flops("NVIDIA OpenCL", s)) {
+                        warnings.push_back(s);
+                    }
                     prop.peak_flops = c.peak_flops;
                 }
                 if (cuda_match_found) {
@@ -552,6 +556,9 @@ void COPROCS::get_opencl(
                     COPROC_ATI c;
                     c.opencl_prop = prop;
                     c.set_peak_flops();
+                    if (c.bad_gpu_peak_flops("AMD OpenCL", s)) {
+                        warnings.push_back(s);
+                    }
                     prop.peak_flops = c.peak_flops;
                 }
 
@@ -576,6 +583,9 @@ void COPROCS::get_opencl(
                 safe_strcpy(c.version, prop.opencl_driver_version);
 
                 c.set_peak_flops();
+                if (c.bad_gpu_peak_flops("Intel OpenCL", s)) {
+                    warnings.push_back(s);
+                }
                 prop.peak_flops = c.peak_flops;
                 prop.opencl_available_ram = prop.global_mem_size;
 
@@ -606,12 +616,22 @@ void COPROCS::get_opencl(
                 prop.opencl_available_ram = prop.global_mem_size;
                 prop.is_used = COPROC_USED;
 
-                // TODO: Find a better way to calculate / estimate peak_flops for future coprocessors?
+                // TODO: is there a better way to estimate peak_flops?
+                //
                 prop.peak_flops = 0;
                 if (prop.max_compute_units) {
-                    prop.peak_flops = prop.max_compute_units * prop.max_clock_frequency * MEGA;
+                    double freq = ((double)prop.max_clock_frequency) * MEGA;
+                    prop.peak_flops = ((double)prop.max_compute_units) * freq;
+                }
+                if (prop.peak_flops <= 0 || prop.peak_flops > GPU_MAX_PEAK_FLOPS) {
+                    char buf2[256];
+                    sprintf(buf2,
+                        "OpenCL generic: bad peak FLOPS; Max units %d, max freq %d MHz",
+                        prop.max_compute_units, prop.max_clock_frequency
+                    );
+                    warnings.push_back(buf2);
+                    prop.peak_flops = GPU_DEFAULT_PEAK_FLOPS;
                 }
-                if (prop.peak_flops <= 0) prop.peak_flops = 45e9;
 
                 other_opencls.push_back(prop);
             }
