@@ -62,7 +62,6 @@ using std::vector;
 
 static CURLM* g_curlMulti = NULL;
 static char g_user_agent_string[256] = {""};
-static const char g_content_type[] = {"Content-Type: application/x-www-form-urlencoded"};
 static unsigned int g_trace_count = 0;
 static bool got_expectation_failed = false;
     // Whether we've got a 417 HTTP error.
@@ -73,11 +72,11 @@ static void get_user_agent_string() {
     if (g_user_agent_string[0]) return;
     snprintf(g_user_agent_string, sizeof(g_user_agent_string),
         "BOINC client (%s %d.%d.%d)",
-        gstate.get_primary_platform(),
+        HOSTTYPE,
         BOINC_MAJOR_VERSION, BOINC_MINOR_VERSION, BOINC_RELEASE
     );
     if (strlen(gstate.client_brand)) {
-        char buf[256];
+        char buf[1024];
         snprintf(buf, sizeof(buf), " (%s)", gstate.client_brand);
         safe_strcat(g_user_agent_string, buf);
     }
@@ -601,11 +600,6 @@ int HTTP_OP::libcurl_exec(
     //
     setup_proxy_session(no_proxy_for_url(url));
 
-
-    // set the content type in the header
-    //
-    pcurlList = curl_slist_append(pcurlList, g_content_type);
-
     if (strlen(gstate.language)) {
         snprintf(buf, sizeof(buf), "Accept-Language: %s", gstate.language);
         pcurlList = curl_slist_append(pcurlList, buf);
@@ -694,7 +688,6 @@ int HTTP_OP::libcurl_exec(
         curl_formadd(&pcurlFormStart, &pcurlFormEnd,
            CURLFORM_FILECONTENT, infile,
            CURLFORM_CONTENTSLENGTH, content_length,
-           CURLFORM_CONTENTTYPE, g_content_type,
            CURLFORM_END
         );
         curl_formadd(&post, &last,
