@@ -451,17 +451,13 @@ void CViewWork::OnWorkSuspend( wxCommandEvent& WXUNUSED(event) ) {
         RESULT* result = pDoc->result(m_iSortedIndexes[row]);
         if (result) {
             if (result->suspended_via_gui) {
-                pFrame->UpdateStatusText(_("Resuming task..."));
                 pDoc->WorkResume(result->project_url, result->name);
             } else {
-                pFrame->UpdateStatusText(_("Suspending task..."));
                 pDoc->WorkSuspend(result->project_url, result->name);
             }
         }
     }
 
-    pFrame->UpdateStatusText(wxT(""));
-    
     UpdateSelection();
     pFrame->FireRefreshView();
 
@@ -483,8 +479,6 @@ void CViewWork::OnWorkShowGraphics( wxCommandEvent& WXUNUSED(event) ) {
     wxASSERT(wxDynamicCast(pFrame, CAdvancedFrame));
     wxASSERT(m_pListPane);
 
-    pFrame->UpdateStatusText(_("Showing graphics for task..."));
-
     row = -1;
     while (1) {
         // Step through all selected items
@@ -497,8 +491,6 @@ void CViewWork::OnWorkShowGraphics( wxCommandEvent& WXUNUSED(event) ) {
         }
     }
 
-    pFrame->UpdateStatusText(wxT(""));
-
     UpdateSelection();
     pFrame->FireRefreshView();
 
@@ -520,8 +512,6 @@ void CViewWork::OnWorkShowVMConsole( wxCommandEvent& WXUNUSED(event) ) {
     wxASSERT(wxDynamicCast(pFrame, CAdvancedFrame));
     wxASSERT(m_pListPane);
 
-    pFrame->UpdateStatusText(_("Showing VM console for task..."));
-
     row = -1;
     while (1) {
         // Step through all selected items
@@ -534,8 +524,6 @@ void CViewWork::OnWorkShowVMConsole( wxCommandEvent& WXUNUSED(event) ) {
         }
     }
 
-    pFrame->UpdateStatusText(wxT(""));
-
     UpdateSelection();
     pFrame->FireRefreshView();
 
@@ -593,8 +581,6 @@ void CViewWork::OnWorkAbort( wxCommandEvent& WXUNUSED(event) ) {
         return;
     }
 
-    pFrame->UpdateStatusText(_("Aborting task..."));
-
     row = -1;
     while (1) {
         // Step through all selected items
@@ -607,8 +593,6 @@ void CViewWork::OnWorkAbort( wxCommandEvent& WXUNUSED(event) ) {
         }
     }
 
-    pFrame->UpdateStatusText(wxT(""));
-
     UpdateSelection();
     pFrame->FireRefreshView();
 
@@ -693,15 +677,11 @@ void CViewWork::OnProjectWebsiteClicked( wxEvent& event ) {
     wxASSERT(m_pTaskPane);
     wxASSERT(m_pListPane);
 
-    pFrame->UpdateStatusText(_("Launching browser..."));
-
     int website_task_index = event.GetId() - ID_TASK_PROJECT_WEB_PROJDEF_MIN;
     wxLaunchDefaultBrowser(
         m_TaskGroups[1]->m_Tasks[website_task_index]->m_strWebSiteLink
     );
 
-    pFrame->UpdateStatusText(wxT(""));
-
     UpdateSelection();
     pFrame->FireRefreshView();
 
