@@ -91,7 +91,8 @@ private:
 };
 
 
-// Define a new frame
+// Simple view frame
+//
 class CSimpleFrame : public CBOINCBaseFrame
 {
     DECLARE_DYNAMIC_CLASS(CSimpleFrame)
@@ -115,8 +116,7 @@ public:
     void OnHelpBOINC( wxCommandEvent& event );
     void OnHelpAbout( wxCommandEvent& event );
     void OnCheckVersion( wxCommandEvent& event );
-
-    void OnProjectsAttachToProject(wxCommandEvent& event);
+    void OnReportBug( wxCommandEvent& event );
 
 	void OnConnect(CFrameEvent& event );
     void OnReloadSkin( CFrameEvent& event );
@@ -132,6 +132,14 @@ public:
     bool RestoreState();
     void OnMove(wxMoveEvent& event);
 
+    void StartTimers() {
+        StartTimersBase();
+    }
+    void StopTimers() {
+        StopTimersBase();
+    }
+    bool CreateMenus();
+
 protected:
     virtual int     _GetCurrentViewPage();
 
