@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2018 University of California
+// Copyright (C) 2019 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -87,6 +87,7 @@ static CScreensaver* gspScreensaver = NULL;
 extern int gGoToBlank;      // True if we are to blank the screen
 extern int gBlankingTime;   // Delay in minutes before blanking the screen
 extern CFStringRef gPathToBundleResources;
+extern RESULT* graphics_app_result_ptr;
 
 static SaverState saverState = SaverState_Idle;
 // int gQuitCounter = 0;
@@ -101,6 +102,11 @@ static bool ScreenIsBlanked = false;
 static int retryCount = 0;
 static pthread_mutexattr_t saver_mutex_attr;
 pthread_mutex_t saver_mutex;
+static char passwd_buf[256];
+bool gIsHighSierra = false;  // OS 10.13 or later
+bool gIsMojave = false;     // OS 10.14 or later
+bool gIsCatalina = false;   // OS 10.15 or later
+bool gUseLaunchAgent = false;
 
 const char *  CantLaunchCCMsg = "Unable to launch BOINC application.";
 const char *  LaunchingCCMsg = "Launching BOINC application.";
@@ -113,7 +119,8 @@ const char *  CantLaunchDefaultGFXAppMsg = "Can't launch default screensaver mod
 const char *  DefaultGFXAppCantRPCMsg = "Default screensaver module couldn't connect to BOINC application";
 const char *  DefaultGFXAppCrashedMsg = "Default screensaver module had an unrecoverable error";
 const char *  RunningOnBatteryMsg = "Computing and screensaver disabled while running on battery power.";
-const char *  IncompatibleMsg = " is not compatible with this version of OS X.";
+const char *  IncompatibleMsg = "Could not connect to screensaver ";
+const char *  CCNotRunningMsg = "BOINC is not running.";
 
 //const char *  BOINCExitedSaverMode = "BOINC is no longer in screensaver mode.";
 
@@ -204,7 +211,7 @@ void incompatibleGfxApp(char * appPath, pid_t pid, int slot){
             
             retval = gspScreensaver->rpc->get_state(gspScreensaver->state);
             if (!retval) {
-                strlcpy(buf, "Screensaver ", sizeof(buf));
+                strlcpy(buf, IncompatibleMsg, sizeof(buf));
                 for (int i=0; i<gspScreensaver->state.results.size(); i++) {
                     RESULT* r = gspScreensaver->state.results[i];
                     if (r->slot == slot) {
@@ -226,7 +233,6 @@ void incompatibleGfxApp(char * appPath, pid_t pid, int slot){
                 strlcat(buf, p+1, sizeof(buf));
                 strlcat(buf, "\"", sizeof(buf));
             }
-            strlcat(buf, IncompatibleMsg, sizeof(buf));
             gspScreensaver->setSSMessageText(buf);
             gspScreensaver->SetError(0, SCRAPPERR_GFXAPPINCOMPATIBLE);
         }   // End if (msgstartTime == 0.0)
@@ -235,7 +241,7 @@ void incompatibleGfxApp(char * appPath, pid_t pid, int slot){
             gspScreensaver->markAsIncompatible(appPath);
             launchedGfxApp("", 0, -1);
             msgstartTime = 0.0;
-            gspScreensaver->terminate_v6_screensaver(pid);
+            gspScreensaver->terminate_v6_screensaver(pid, graphics_app_result_ptr);
         }
     }
 }
@@ -298,6 +304,7 @@ void doBoinc_Sleep(double seconds) {
 
 CScreensaver::CScreensaver() {
     struct ss_periods periods;
+    char saved_dir[MAXPATHLEN];
     
     m_dwBlankScreen = 0;
     m_dwBlankTime = 0;
@@ -318,6 +325,15 @@ CScreensaver::CScreensaver() {
     m_bResetCoreState = true;
     rpc = 0;
     m_bConnected = false;
+    m_gfx_Cleanup_IPC = NULL;
+    safe_strcpy(passwd_buf, "");
+   
+    if (gUseLaunchAgent) {
+        getcwd(saved_dir, sizeof(saved_dir));
+        chdir("/Library/Application Support/BOINC Data");
+        read_gui_rpc_password(passwd_buf);
+        chdir(saved_dir);
+    }
     
     // Get project-defined default values for GFXDefaultPeriod, GFXSciencePeriod, GFXChangePeriod
     GetDefaultDisplayPeriods(periods);
@@ -379,8 +395,19 @@ int CScreensaver::Create() {
     // against launching multiple instances of the core client
     if (saverState == SaverState_Idle) {
         CFStringGetCString(gPathToBundleResources, m_gfx_Switcher_Path, sizeof(m_gfx_Switcher_Path), kCFStringEncodingMacRoman);
+        strlcpy(m_gfx_Cleanup_Path, "\"", sizeof(m_gfx_Cleanup_Path));
+        strlcat(m_gfx_Cleanup_Path, m_gfx_Switcher_Path, sizeof(m_gfx_Cleanup_Path));
         strlcat(m_gfx_Switcher_Path, "/gfx_switcher", sizeof(m_gfx_Switcher_Path));
+        strlcat(m_gfx_Cleanup_Path, "/gfx_cleanup\"", sizeof(m_gfx_Switcher_Path));
 
+        if (gUseLaunchAgent) {
+            // Launch helper app to work around a bug in OS 10.15 Catalina to
+            // kill current graphics app if ScreensaverEngine exits without 
+            // first calling [ScreenSaverView stopAnimation]
+            //TODO: Should we use this on OS 10.13+ ?
+            m_gfx_Cleanup_IPC = popen(m_gfx_Cleanup_Path, "w");
+        }
+        
         err = initBOINCApp();
 
         CGDisplayHideCursor(kCGNullDirectDisplay);
@@ -416,7 +443,7 @@ OSStatus CScreensaver::initBOINCApp() {
 
     saverState = SaverState_CantLaunchCoreClient;
     
-    m_CoreClientPID = FindProcessPID("boinc", 0);
+    m_CoreClientPID = getClientPID();
     if (m_CoreClientPID) {
         m_wasAlreadyRunning = true;
         saverState = SaverState_LaunchingCoreClient;
@@ -426,6 +453,9 @@ OSStatus CScreensaver::initBOINCApp() {
     
     m_wasAlreadyRunning = false;
     
+    if (gIsCatalina) {
+        return noErr;   // Screensavers can't launch setuid /setgid processes as of Catalina
+    }
     if (++retryCount > 3)   // Limit to 3 relaunches to prevent thrashing
         return -1;
 
@@ -478,6 +508,7 @@ OSStatus CScreensaver::initBOINCApp() {
 
 // Returns new desired Animation Frequency (per second) or 0 for no change
 int CScreensaver::getSSMessage(char **theMessage, int* coveredFreq) {
+    int retval;
     int newFrequency = TEXTLOGOFREQUENCY;
     *coveredFreq = 0;
     pid_t myPid;
@@ -504,11 +535,20 @@ int CScreensaver::getSSMessage(char **theMessage, int* coveredFreq) {
             setSSMessageText(LaunchingCCMsg);
         }
             
-        myPid = FindProcessPID(NULL, m_CoreClientPID);
+        myPid = getClientPID();
         if (myPid) {
             saverState = SaverState_CoreClientRunning;
+
             if (!rpc->init(NULL)) {     // Initialize communications with Core Client
                 m_bConnected = true;
+                
+                if (strlen(passwd_buf)) {
+                    retval = rpc->authorize(passwd_buf);
+                    if (retval) {
+                        fprintf(stderr, "Screensaver RPC authorization failure: %d\n", retval);
+                    }
+                }
+
                 if (IsDualGPUMacbook) {
                     ccstate.clear();
                     ccstate.global_prefs.init_bools();
@@ -529,13 +569,17 @@ int CScreensaver::getSSMessage(char **theMessage, int* coveredFreq) {
             // and running screensaver graphics
             CreateDataManagementThread();
             // ToDo: Add a timeout after which we display error message
-        } else
+        } else {
+            if (gIsCatalina) {
+                return noErr;   // Screensavers can't launch setuid /setgid processes as of Catalina
+            }
             // Take care of the possible race condition where the Core Client was in the  
             // process of shutting down just as ScreenSaver started, so initBOINCApp() 
             // found it already running but now it has shut down.
             if (m_wasAlreadyRunning) { // If we launched it, then just wait for it to start
                 saverState = SaverState_RelaunchCoreClient;
             }
+        }
          break;
 
     case SaverState_CoreClientRunning:
@@ -642,7 +686,12 @@ int CScreensaver::getSSMessage(char **theMessage, int* coveredFreq) {
             break;
         }
         
-        setSSMessageText(CantLaunchCCMsg);
+        if (gIsCatalina) {
+            setSSMessageText(CCNotRunningMsg);
+            break;
+        } else {
+            setSSMessageText(CantLaunchCCMsg);
+        }
         
         // Set up a separate thread for running screensaver graphics 
         // even if we can't communicate with core client
@@ -708,6 +757,11 @@ void CScreensaver::ShutdownSaver() {
     m_bQuitDataManagementProc = false;
     saverState = SaverState_Idle;
     retryCount = 0;
+    if (m_gfx_Cleanup_IPC) {
+        fprintf(m_gfx_Cleanup_IPC, "Quit\n");
+        fflush(m_gfx_Cleanup_IPC);
+        pclose(m_gfx_Cleanup_IPC);
+    }
 }
 
 
@@ -722,12 +776,16 @@ void * CScreensaver::DataManagementProcStub(void* param) {
 void CScreensaver::HandleRPCError() {
     static time_t last_RPC_retry = 0;
     time_t now = time(0);
+    int retval;
    
     // Attempt to restart BOINC Client if needed, reinitialize the RPC client and state
     rpc->close();
     m_bConnected = false;
     
     if (saverState == SaverState_CantLaunchCoreClient) {
+        if (gIsCatalina) {
+            return;   // Screensavers can't launch setuid /setgid processes as of Catalina
+        }
         if ((now - last_RPC_retry) < RPC_RETRY_INTERVAL) {
             return;
         }
@@ -738,7 +796,7 @@ void CScreensaver::HandleRPCError() {
         // found it already running but now it has shut down.  This code takes 
         // care of that and other situations where the Core Client quits unexpectedy.  
         // Code in initBOINC_App() limits # launch retries to 3 to prevent thrashing.
-        if (FindProcessPID("boinc", 0) == 0) {
+        if (getClientPID() == 0) {
             saverState = SaverState_RelaunchCoreClient;
             m_bResetCoreState = true;
          }
@@ -754,6 +812,12 @@ void CScreensaver::HandleRPCError() {
     // Otherwise just reinitialize the RPC client and state and keep trying
     if (!rpc->init(NULL)) {
         m_bConnected = true;
+        if (strlen(passwd_buf)) {
+            retval = rpc->authorize(passwd_buf);
+            if (retval) {
+                fprintf(stderr, "Screensaver RPC authorization failure: %d\n", retval);
+            }
+        }
     }
     // Error message after timeout?
 }
@@ -798,15 +862,16 @@ bool CScreensaver::DestroyDataManagementThread() {
         boinc_sleep(0.1);
     }
 
-    if (rpc) {
-        rpc->close();    // In case DataManagementProc is hung waiting for RPC
-    }
     m_hDataManagementThread = NULL; // Don't delay more if this routine is called again.
+
     if (m_hGraphicsApplication) {
-        terminate_v6_screensaver(m_hGraphicsApplication);
+        terminate_v6_screensaver(m_hGraphicsApplication, graphics_app_result_ptr);
         m_hGraphicsApplication = 0;
     }
 
+    if (rpc) {
+        rpc->close();    // In case DataManagementProc is hung waiting for RPC
+    }
     return true;
 }
 
@@ -887,58 +952,27 @@ int CScreensaver::GetBrandID()
 }
 
 
-char * CScreensaver::PersistentFGets(char *buf, size_t buflen, FILE *f) {
-    char *p = buf;
-    size_t len = buflen;
-    size_t datalen = 0;
-
-    *buf = '\0';
-    while (datalen < (buflen - 1)) {
-        fgets(p, len, f);
-        if (feof(f)) break;
-        if (ferror(f) && (errno != EINTR)) break;
-        if (strchr(buf, '\n')) break;
-        datalen = strlen(buf);
-        p = buf + datalen;
-        len -= datalen;
+pid_t CScreensaver::getClientPID() {
+    int fd;
+    fd = open("/Library/Application Support/BOINC Data/lockfile", O_RDONLY);
+    if (fd<0) {
+        return 0;   // lockfile doesn't exist (probably)
     }
-    return (buf[0] ? buf : NULL);
-}
-
-
-pid_t CScreensaver::FindProcessPID(char* name, pid_t thePID)
-{
-    FILE *f;
-    char buf[1024];
-    size_t n = 0;
-    pid_t aPID;
-    
-    if (name != NULL)     // Search ny name
-        n = strlen(name);
-    
-    f = popen("ps -a -x -c -o command,pid", "r");
-    if (f == NULL)
-        return 0;
-    
-    while (PersistentFGets(buf, sizeof(buf), f))
-    {
-        if (name != NULL) {     // Search ny name
-            if (strncmp(buf, name, n) == 0)
-            {
-                aPID = atol(buf+16);
-                pclose(f);
-                return aPID;
-            }
-        } else {      // Search by PID
-            aPID = atol(buf+16);
-            if (aPID == thePID) {
-                pclose(f);
-                return aPID;
-            }
-        }
+    struct flock fl;
+    fl.l_type = F_WRLCK;
+    fl.l_pid = 0;
+    fl.l_whence = SEEK_SET;
+    fl.l_start = 0;
+    fl.l_len = 0;
+    // fcntl F_GETLK sets fl.l_pid to PID of lock's owner if the
+    // file is locked, else leaves it unchanged
+    if (fcntl(fd, F_GETLK, &fl) != 0) {
+        close(fd);
+        return 0;   // fcntl failed (should never happen)
     }
-    pclose(f);
-    return 0;
+    
+    close(fd);
+    return fl.l_pid;
 }
 
 
@@ -1071,7 +1105,8 @@ void print_to_log_file(const char *format, ...) {
     char buf[256];
     time_t t;
 #if USE_SPECIAL_LOG_FILE
-    safe_strcpy(buf, getenv("HOME"));
+    safe_strcpy(buf, "/Users/");
+    safe_strcat(buf, getenv("USER"));
     safe_strcat(buf, "/Documents/test_log.txt");
     FILE *f;
     f = fopen(buf, "a");
