@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// Copyright (C) 2019 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -112,6 +112,7 @@ void BADGE_USER::clear() {memset(this, 0, sizeof(*this));}
 void BADGE_TEAM::clear() {memset(this, 0, sizeof(*this));}
 void CREDIT_USER::clear() {memset(this, 0, sizeof(*this));}
 void CREDIT_TEAM::clear() {memset(this, 0, sizeof(*this));}
+void CONSENT_TYPE::clear() {memset(this, 0, sizeof(*this));}
 
 DB_PLATFORM::DB_PLATFORM(DB_CONN* dc) :
     DB_BASE("platform", dc?dc:&boinc_db){}
@@ -131,6 +132,8 @@ DB_HOST_DELETED::DB_HOST_DELETED(DB_CONN* dc) :
     DB_BASE("host_deleted", dc?dc:&boinc_db){}
 DB_WORKUNIT::DB_WORKUNIT(DB_CONN* dc) :
     DB_BASE("workunit", dc?dc:&boinc_db){}
+DB_BATCH::DB_BATCH(DB_CONN* dc) :
+    DB_BASE("batch", dc?dc:&boinc_db){}
 DB_CREDITED_JOB::DB_CREDITED_JOB(DB_CONN* dc) :
     DB_BASE("credited_job", dc?dc:&boinc_db){}
 DB_RESULT::DB_RESULT(DB_CONN* dc) :
@@ -148,9 +151,13 @@ DB_USER_SUBMIT::DB_USER_SUBMIT(DB_CONN* dc) :
 DB_STATE_COUNTS::DB_STATE_COUNTS(DB_CONN* dc) :
     DB_BASE("state_counts", dc?dc:&boinc_db){}
 DB_TRANSITIONER_ITEM_SET::DB_TRANSITIONER_ITEM_SET(DB_CONN* dc) :
-    DB_BASE_SPECIAL(dc?dc:&boinc_db){}
+    DB_BASE_SPECIAL(dc?dc:&boinc_db) {
+    nitems_this_query = 0;
+}
 DB_VALIDATOR_ITEM_SET::DB_VALIDATOR_ITEM_SET(DB_CONN* dc) :
-    DB_BASE_SPECIAL(dc?dc:&boinc_db){}
+    DB_BASE_SPECIAL(dc?dc:&boinc_db) {
+    nitems_this_query = 0;
+}
 DB_WORK_ITEM::DB_WORK_ITEM(DB_CONN* dc) :
     DB_BASE_SPECIAL(dc?dc:&boinc_db
 ){
@@ -193,6 +200,8 @@ DB_CREDIT_USER::DB_CREDIT_USER(DB_CONN* dc) :
     DB_BASE("credit_user", dc?dc:&boinc_db){}
 DB_CREDIT_TEAM::DB_CREDIT_TEAM(DB_CONN* dc) :
     DB_BASE("credit_team", dc?dc:&boinc_db){}
+DB_CONSENT_TYPE::DB_CONSENT_TYPE(DB_CONN* dc) :
+    DB_BASE("consent_type", dc?dc:&boinc_db){}
 
 DB_ID_TYPE DB_PLATFORM::get_id() {return id;}
 DB_ID_TYPE DB_APP::get_id() {return id;}
@@ -203,6 +212,7 @@ DB_ID_TYPE DB_TEAM::get_id() {return id;}
 DB_ID_TYPE DB_HOST::get_id() {return id;}
 DB_ID_TYPE DB_HOST_DELETED::get_id() {return hostid;}
 DB_ID_TYPE DB_WORKUNIT::get_id() {return id;}
+DB_ID_TYPE DB_BATCH::get_id() {return id;}
 DB_ID_TYPE DB_RESULT::get_id() {return id;}
 DB_ID_TYPE DB_MSG_FROM_HOST::get_id() {return id;}
 DB_ID_TYPE DB_MSG_TO_HOST::get_id() {return id;}
@@ -212,6 +222,7 @@ DB_ID_TYPE DB_FILE::get_id() {return id;}
 DB_ID_TYPE DB_FILESET::get_id() {return id;}
 DB_ID_TYPE DB_SCHED_TRIGGER::get_id() {return id;}
 DB_ID_TYPE DB_VDA_FILE::get_id() {return id;}
+DB_ID_TYPE DB_CONSENT_TYPE::get_id() {return id;}
 
 void DB_PLATFORM::db_print(char* buf){
     sprintf(buf,
@@ -2865,4 +2876,30 @@ void DB_CREDIT_TEAM::db_parse(MYSQL_ROW &r) {
     credit_type = atoi(r[i++]);
 }
 
-const char *BOINC_RCSID_ac374386c8 = "$Id$";
+void DB_CONSENT_TYPE::db_print(char *buf) {
+    sprintf(buf,
+	"id=%lu, "
+	"shortname='%s', "
+	"description='%s', "
+	"enabled=%d, "
+	"project_specific=%d, "
+	"privacypref=%d, ",
+	id,
+	shortname,
+	description,
+	enabled,
+	project_specific,
+	privacypref
+    );
+}
+
+void DB_CONSENT_TYPE::db_parse(MYSQL_ROW &r) {
+    int i=0;
+    clear();
+    id = atol(r[i++]);
+    strcpy2(shortname, r[i++]);
+    strcpy2(description, r[i++]);
+    enabled = atoi(r[i++]);
+    project_specific = atoi(r[i++]);
+    privacypref = atoi(r[i++]);
+}
