@@ -181,3 +181,21 @@ alter table post_ratings
 alter table sent_email
     add index sent_email_userid(userid);
         -- for delete account
+
+alter table private_messages
+    add index userid(userid);
+
+alter table consent
+    add index userid_ctid_timestamp(userid, consent_type_id, consent_time),
+    add index consent_timestamp(consent_time),
+    add index flag_ctid(consent_flag, consent_type_id);
+
+alter table consent
+    add foreign key(consent_type_id)
+    references consent_type(id)
+    on update cascade
+    on delete restrict;
+      -- explicit delete restrict on this foreign key
+
+alter table consent_type
+    add index consent_name(shortname);
