@@ -1,10 +1,9 @@
 <?php
 /**
  * @file
- * Template for a 3 column panel layout.
+ * Template for the 1 column panel layout.
  *
- * This template provides a three column panel display layout, with
- * each column roughly equal in width.
+ * This template provides a three column 25%-50%-25% panel display layout.
  *
  * Variables:
  * - $id: An optional CSS id to use for the layout.
@@ -15,8 +14,7 @@
  *   - $content['right']: Content in the right column.
  */
 ?>
-
-<div class="panel-display panel-3col-33 clear-block" <?php if (!empty($css_id)) { print "id=\"$css_id\""; } ?>>
+<div class="panel-display panel-3col clear-block" <?php if (!empty($css_id)) { print "id=\"$css_id\""; } ?>>
   <div class="panel-panel panel-col-first">
     <div class="inside"><?php print $content['left']; ?></div>
   </div>
