@@ -29,18 +29,25 @@ require_once("../inc/submit_util.inc");
 error_reporting(E_ALL);
 ini_set('display_errors', true);
 ini_set('display_startup_errors', true);
+ini_set("memory_limit", "4G");
 
 function get_wu($name) {
     $name = BoincDb::escape_string($name);
     $wu = BoincWorkunit::lookup("name='$name'");
-    if (!$wu) xml_error(-1, "no job named $name was found");
+    if (!$wu) {
+        log_write("no job named $name was found");
+        xml_error(-1, "no job named $name was found");
+    }
     return $wu;
 }
 
 function get_submit_app($name) {
     $name = BoincDb::escape_string($name);
     $app = BoincApp::lookup("name='$name'");
-    if (!$app) xml_error(-1, "no app named $name was found");
+    if (!$app) {
+        log_write("no app named $name was found");
+        xml_error(-1, "no app named $name was found");
+    }
     return $app;
 }
 
@@ -61,6 +68,7 @@ function batch_flop_count($r, $template) {
         } else if ($t) {
             $x += $t;
         } else {
+            log_write("no rsc_fpops_est given for job");
             xml_error(-1, "no rsc_fpops_est given for job");
         }
     }
@@ -96,6 +104,7 @@ function read_input_template($app, $r) {
     if (file_exists($path)) {
         $x = simplexml_load_file($path);
         if (!$x) {
+            log_write("couldn't parse input template file $path");
             xml_error(-1, "couldn't parse input template file $path");
         }
         return $x;
@@ -111,6 +120,7 @@ function check_max_jobs_in_progress($r, $user_submit) {
     $n = $db->get_int($query, 'total');
     if ($n === false) return;
     if ($n + count($r->batch->job) > $user_submit->max_jobs_in_progress) {
+        log_write("limit on jobs in progress exceeded");
         xml_error(-1, "limit on jobs in progress exceeded");
     }
 }
@@ -137,6 +147,7 @@ function validate_batch($jobs, $template) {
     foreach($jobs as $job) {
         $m = count($job->input_files);
         if ($n != $m) {
+            log_write("wrong # of input files for job $i: need $n, got $m");
             xml_error(-1, "wrong # of input files for job $i: need $n, got $m");
         }
         $i++;
@@ -159,12 +170,14 @@ function stage_file($file) {
         //
         $md5 = md5_file($file->source);
         if (!$md5) {
+            log_write("Can't get MD5 of file $file->source");
             xml_error(-1, "Can't get MD5 of file $file->source");
         }
         $name = job_file_name($md5);
         $path = dir_hier_path($name, $download_dir, $fanout);
         if (file_exists($path)) return $name;
         if (!copy($file->source, $path)) {
+            log_write("can't copy file from $file->source to $path");
             xml_error(-1, "can't copy file from $file->source to $path");
         }
         return $name;
@@ -173,16 +186,19 @@ function stage_file($file) {
     case "inline":
         $md5 = md5($file->source);
         if (!$md5) {
+            log_write("Can't get MD5 of inline data");
             xml_error(-1, "Can't get MD5 of inline data");
         }
         $name = job_file_name($md5);
         $path = dir_hier_path($name, $download_dir, $fanout);
         if (file_exists($path)) return $name;
         if (!file_put_contents($path, $file->source)) {
+            log_write("can't write to file $path");
             xml_error(-1, "can't write to file $path");
         }
         return $name;
     }
+    log_write(-1, "unsupported file mode: $file->mode");
     xml_error(-1, "unsupported file mode: $file->mode");
 }
 
@@ -394,7 +410,6 @@ function submit_batch($r) {
         validate_batch($jobs, $template);
     }
     stage_files($jobs);
-    $njobs = count($jobs);
     $now = time();
     $app_version_num = (int)($r->batch->app_version_num);
 
@@ -405,12 +420,15 @@ function submit_batch($r) {
     if ($batch_id) {
         $batch = BoincBatch::lookup_id($batch_id);
         if (!$batch) {
+            log_write("not batch $batch_id");
             xml_error(-1, "no batch $batch_id");
         }
         if ($batch->user_id != $user->id) {
+            log_write("not owner of batch");
             xml_error(-1, "not owner of batch");
         }
         if ($batch->state != BATCH_STATE_INIT) {
+            log_write("batch not in init state");
             xml_error(-1, "batch not in init state");
         }
     }
@@ -433,6 +451,7 @@ function submit_batch($r) {
             if ($x) {
                 $total_flops += $x;
             } else {
+                log_write("no rsc_fpops_est given");
                 xml_error(-1, "no rsc_fpops_est given");
             }
         }
@@ -440,14 +459,19 @@ function submit_batch($r) {
     $cmd = "cd " . project_dir() . "/bin; ./adjust_user_priority --user $user->id --flops $total_flops --app $app->name";
     $x = exec($cmd);
     if (!is_numeric($x) || (double)$x == 0) {
+        log_write("$cmd returned $x");
         xml_error(-1, "$cmd returned $x");
     }
     $let = (double)$x;
 
+    $njobs = count($jobs);
     if ($batch_id) {
-        $njobs = count($jobs);
         $ret = $batch->update("njobs=$njobs, logical_end_time=$let");
-        if (!$ret) xml_error(-1, "batch->update() failed");
+        if (!$ret) {
+            log_write("batch update to njobs failed");
+            xml_error(-1, "batch->update() failed");
+        }
+        log_write("adding jobs to existing batch $batch_id");
     } else {
         $batch_name = (string)($r->batch->batch_name);
         $batch_name = BoincDb::escape_string($batch_name);
@@ -455,9 +479,11 @@ function submit_batch($r) {
             "(user_id, create_time, njobs, name, app_id, logical_end_time, state) values ($user->id, $now, $njobs, '$batch_name', $app->id, $let, ".BATCH_STATE_INIT.")"
         );
         if (!$batch_id) {
+            log_write("can't create batch");
             xml_error(-1, "Can't create batch: ".BoincDb::error());
         }
         $batch = BoincBatch::lookup_id($batch_id);
+        log_write("created batch $batch_id");
     }
     
     $job_params = new StdClass;
@@ -479,10 +505,14 @@ function submit_batch($r) {
     );
 
     // set state to IN_PROGRESS only after creating jobs;
-    // otherwise we might flag batch as COMPLETED
+    // otherwise something else might flag batch as COMPLETE
     //
     $ret = $batch->update("state= ".BATCH_STATE_IN_PROGRESS);
-    if (!$ret) xml_error(-1, "batch->update() failed");
+    if (!$ret) {
+        log_write("batch update to IN_PROGRESS failed");
+        xml_error(-1, "batch->update() failed");
+    }
+    log_write("updated batch state to IN_PROGRESS");
 
     echo "<batch_id>$batch_id</batch_id>
         </submit_batch>
@@ -503,6 +533,7 @@ function create_batch($r) {
         "(user_id, create_time, name, app_id, state, expire_time) values ($user->id, $now, '$batch_name', $app->id, ".BATCH_STATE_INIT.", $expire_time)"
     );
     if (!$batch_id) {
+        log_write("Can't create batch: ".BoincDb::error());
         xml_error(-1, "Can't create batch: ".BoincDb::error());
     }
     echo "<batch_id>$batch_id</batch_id>
@@ -625,9 +656,13 @@ function get_batch($r) {
         $batch_name = BoincDb::escape_string($batch_name);
         $batch = BoincBatch::lookup_name($batch_name);
     } else {
+        log_write("batch not specified");
         xml_error(-1, "batch not specified");
     }
-    if (!$batch) xml_error(-1, "no such batch");
+    if (!$batch) {
+        log_write("no such batch");
+        xml_error(-1, "no such batch");
+    }
     return $batch;
 }
 
@@ -636,6 +671,7 @@ function query_batch($r) {
     list($user, $user_submit) = authenticate_user($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
+        log_write("not owner of batch");
         xml_error(-1, "not owner of batch");
     }
 
@@ -682,9 +718,11 @@ function query_batch2($r) {
         $batch_name = BoincDb::escape_string($batch_name);
         $batch = BoincBatch::lookup_name($batch_name);
         if (!$batch) {
+            log_write("no batch named $batch_name");
             xml_error(-1, "no batch named $batch_name");
         }
         if ($batch->user_id != $user->id) {
+            log_write("not owner of $batch_name");
             xml_error(-1, "not owner of $batch_name");
         }
         $batches[] = $batch;
@@ -734,9 +772,13 @@ function query_job($r) {
     list($user, $user_submit) = authenticate_user($r, null);
     $job_id = (int)($r->job_id);
     $wu = BoincWorkunit::lookup_id($job_id);
-    if (!$wu) xml_error(-1, "no such job");
+    if (!$wu) {
+        log_write("no such job");
+        xml_error(-1, "no such job");
+    }
     $batch = BoincBatch::lookup_id($wu->batch);
     if ($batch->user_id != $user->id) {
+        log_write("not owner");
         xml_error(-1, "not owner");
     }
     $results = BoincResult::enum("workunitid=$job_id");
@@ -774,9 +816,13 @@ function query_completed_job($r) {
     $job_name = (string)($r->job_name);
     $job_name = BoincDb::escape_string($job_name);
     $wu = BoincWorkunit::lookup("name='$job_name'");
-    if (!$wu) xml_error(-1, "no such job");
+    if (!$wu) {
+        log_write("no such job");
+        xml_error(-1, "no such job");
+    }
     $batch = BoincBatch::lookup_id($wu->batch);
     if ($batch->user_id != $user->id) {
+        log_write("not owner");
         xml_error(-1, "not owner");
     }
 
@@ -818,6 +864,7 @@ function handle_abort_batch($r) {
     list($user, $user_submit) = authenticate_user($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
+        log_write("not owner");
         xml_error(-1, "not owner");
     }
     abort_batch($batch);
@@ -836,15 +883,18 @@ function handle_abort_jobs($r) {
         $job_name = BoincDb::escape_string($job_name);
         $wu = BoincWorkunit::lookup("name='$job_name'");
         if (!$wu) {
+            log_write("no job $job_name");
             xml_error(-1, "no job $job_name");
         }
         if (!$wu->batch) {
+            log_write("job $job_name is not part of a batch");
             xml_error(-1, "job $job_name is not part of a batch");
         }
         if (!$batch || $wu->batch != $batch->id) {
             $batch = BoincBatch::lookup_id($wu->batch);
         }
         if (!$batch || $batch->user_id != $user->id) {
+            log_write("not owner of batch");
             xml_error(-1, "not owner of batch");
         }
         echo "<aborted $job_name>\n";
@@ -860,6 +910,7 @@ function handle_retire_batch($r) {
     list($user, $user_submit) = authenticate_user($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
+        log_write("not owner of batch");
         xml_error(-1, "not owner of batch");
     }
     retire_batch($batch);
@@ -873,12 +924,14 @@ function handle_set_expire_time($r) {
     list($user, $user_submit) = authenticate_user($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
+        log_write("not owner of batch");
         xml_error(-1, "not owner of batch");
     }
     $expire_time = (double)($r->expire_time);
     if ($batch->update("expire_time=$expire_time")) {
         echo "<success>1</success>";
     } else {
+        log_write("batch update failed");
         xml_error(-1, "batch update failed");
     }
     echo "</set_expire_time>\n";
@@ -899,6 +952,7 @@ function get_templates($r) {
     $in = file_get_contents(project_dir() . "/templates/".$app->name."_in");
     $out = file_get_contents(project_dir() . "/templates/".$app->name."_out");
     if ($in === false || $out === false) {
+        log_write("template file missing");
         xml_error(-1, "template file missing");
     }
     echo "<templates>\n$in\n$out\n</templates>
@@ -970,13 +1024,13 @@ estimate_batch($r);
 exit;
 }
 
-$request_log = parse_config(get_config(), "<remote_submission_log>");
+// optionally write request message (XML) to log file
+//
+$request_log = parse_config(get_config(), "<remote_submit_request_log>");
 if ($request_log) {
-    $request_log_dir = parse_config(get_config(), "<log_dir>");
-    if ($request_log_dir) {
-        $request_log = $request_log_dir . "/" . $request_log;
-    }
-    if ($file = fopen($request_log, "a+")) {
+    $log_dir = parse_config(get_config(), "<log_dir>");
+    $request_log = $log_dir . "/" . $request_log;
+    if ($file = fopen($request_log, "a")) {
         fwrite($file, "\n<submit_rpc_handler date=\"" . date(DATE_ATOM) . "\">\n" . $_POST['request'] . "\n</submit_rpc_handler>\n");
         fclose($file);
     }
@@ -990,9 +1044,12 @@ if (0) {
 }
 $r = simplexml_load_string($req);
 if (!$r) {
+    log_write("----- RPC request: can't parse request message: $req");
     xml_error(-1, "can't parse request message: $req");
 }
 
+log_write("----- Handling RPC; command ".$r->getName());
+
 switch ($r->getName()) {
     case 'abort_batch': handle_abort_batch($r); break;
     case 'abort_jobs': handle_abort_jobs($r); break;
@@ -1008,7 +1065,12 @@ switch ($r->getName()) {
     case 'retire_batch': handle_retire_batch($r); break;
     case 'set_expire_time': handle_set_expire_time($r); break;
     case 'submit_batch': submit_batch($r); break;
-    default: xml_error(-1, "bad command: ".$r->getName());
+    default:
+        log_write("bad command");
+        xml_error(-1, "bad command: ".$r->getName());
+        break;
 }
 
+log_write("RPC done");
+
 ?>
