@@ -1052,7 +1052,7 @@ void CC_STATE::clear() {
     results.clear();
     platforms.clear();
     executing_as_daemon = false;
-    host_info.clear();
+    host_info.clear_host_info();
     have_nvidia = false;
     have_ati = false;
 }
