@@ -38,14 +38,6 @@
 
 #include "prefs.h"
 
-GLOBAL_PREFS_MASK::GLOBAL_PREFS_MASK() {
-    clear();
-}
-
-void GLOBAL_PREFS_MASK::clear() {
-    memset(this, 0, sizeof(GLOBAL_PREFS_MASK));
-}
-
 void GLOBAL_PREFS_MASK::set_all() {
     battery_charge_min_pct = true;
     battery_max_temperature = true;
@@ -267,7 +259,6 @@ void GLOBAL_PREFS::defaults() {
     // mod_time, host_specific here
     // since they are outside of <venue> elements,
     // and this is called when find the right venue.
-    // Also, don't memset to 0
 }
 
 // values for fields with an enabling checkbox in the GUI.
