@@ -616,6 +616,7 @@ const char* boincerror(int which_error) {
         case ERR_CHMOD : return "chmod() failed";
         case ERR_STAT : return "stat() failed";
         case ERR_FCLOSE : return "fclose() failed";
+        case ERR_INVALID_STATE: return "invalid state";
         case HTTP_STATUS_NOT_FOUND: return "HTTP file not found";
         case HTTP_STATUS_PROXY_AUTH_REQ: return "HTTP proxy authentication failure";
         case HTTP_STATUS_RANGE_REQUEST_ERROR: return "HTTP range request error";
@@ -625,6 +626,7 @@ const char* boincerror(int which_error) {
         case HTTP_STATUS_BAD_GATEWAY: return "HTTP bad gateway";
         case HTTP_STATUS_SERVICE_UNAVAILABLE: return "HTTP service unavailable";
         case HTTP_STATUS_GATEWAY_TIMEOUT: return "HTTP gateway timeout";
+        case ERR_ACCT_REQUIRE_CONSENT: return "This project requires to consent to its terms of use";
     }
     static char buf[128];
     sprintf(buf, "Error %d", which_error);
