@@ -427,16 +427,17 @@ int run_program(
     memset(&startup_info, 0, sizeof(startup_info));
     startup_info.cb = sizeof(startup_info);
 
-    safe_strcpy(cmdline, "");
-    for (int i=0; i<argc; i++) {
+    // lpApplicationName needs to be NULL for CreateProcess to search path
+    // but argv[0] may be full path or just filename
+    // 'file' should be something runnable so use that as program name
+    snprintf(cmdline, sizeof(cmdline), "\"%s\"", file);
+    for (int i=1; i<argc; i++) {
+        safe_strcat(cmdline, " ");
         safe_strcat(cmdline, argv[i]);
-        if (i<argc-1) {
-            safe_strcat(cmdline, " ");
-        }
     }
 
     retval = CreateProcessA(
-        file,
+        NULL,
         cmdline,
         NULL,
         NULL,
@@ -484,12 +485,13 @@ int run_program(
         FCGI::perror("execvp");
 #else
         perror("execvp");
+        fprintf(stderr, "couldn't exec %s: %d\n", file, errno);
 #endif
         exit(errno);
     }
 
     if (nsecs) {
-        boinc_sleep(3);
+        boinc_sleep(nsecs);
         if (waitpid(pid, 0, WNOHANG) == pid) {
             return -1;
         }
