@@ -36,7 +36,7 @@ def init():
         fatal_error("Invalid install method: %s"%options.install_method)
 
 def verbose_echo(level, line):
-    print line
+    print(line)
     sys.stdout.flush()
 
 def fatal_error(msg):
@@ -59,7 +59,7 @@ def verbose_sleep(msg, wait):
 def get_env_var(name, default = None):
     value = os.environ.get(name, default)
     if value == None:
-        print "Environment variable %s not defined" % name
+        print("Environment variable %s not defined" % name)
         sys.exit(1)
     return value
 
@@ -89,7 +89,7 @@ def my_symlink(src,dest):
     dest = destpath(src,dest)
     try:
         os.symlink(src,dest)
-    except OSError, e:
+    except OSError as e:
         e.filename = src + ' -> ' + dest
         raise
 
@@ -97,7 +97,7 @@ def my_link(src,dest):
     dest = destpath(src,dest)
     try:
         os.link(src,dest)
-    except OSError, e:
+    except OSError as e:
         e.filename = src + ' -> ' + dest
         raise
 
@@ -108,7 +108,7 @@ def install(src, dest, unless_exists=False):
     try:
         options.install_function(src, dest)
     except:
-        print 'failed to copy ' + src + ' to ' + dest
+        print('failed to copy ' + src + ' to ' + dest)
         return
 
 
@@ -167,11 +167,11 @@ def _url_to_filename(url):
 def account_file_name(url):
     return 'account_' + _url_to_filename(url) + '.xml'
 
-def srcdir(*dirs):
-    return apply(os.path.join,(options.srcdir,)+dirs)
-
-def builddir(*dirs):
-    return apply(os.path.join,(boinc_path_config.TOP_BUILD_DIR,)+dirs)
+def srcdir(location):
+    return os.path.join(options.srcdir, location)
+    
+def builddir(location):
+    return os.path.join(boinc_path_config.TOP_BUILD_DIR, location)
 
 def run_tool(cmd):
     verbose_shell_call(builddir('tools', cmd))
@@ -237,52 +237,51 @@ def num_wus_assimilated():
 def num_wus_to_transition():
     return database.Workunits.count(_extra_params = ['transition_time<%d'%(time.time()+30*86400)])
 
-def build_command_line(cmd, **kwargs):
-    for (key, value) in kwargs.items():
+def build_command_line(cmd, kwargs):
+    for (key, value) in kwargs:
         cmd += " -%s '%s'" %(key,value)
     return cmd
 
 def create_project_dirs(dest_dir):
-    def dir(*d):
-        return apply(os.path.join,(dest_dir,)+d)
     def mkdir2(d):
         try:
             os.makedirs(d)
         except OSError as e:
             if not os.path.isdir(d):
                 raise SystemExit(e)
-    map(lambda d: mkdir2(dir(d)),
-        [   '',
-            'cgi-bin',
-            'bin',
-            'py',
-            'py/Boinc',
-            'templates',
-            'upload',
-            'download',
-            'apps',
-            'html',
-            'html/cache',
-            'html/inc',
-            'html/inc/password_compat',
-            'html/inc/random_compat',
-            'html/inc/ReCaptcha',
-            'html/inc/ReCaptcha/RequestMethod',
-            'html/languages',
-            'html/languages/compiled',
-            'html/languages/translations',
-            'html/languages/project_specific_translations',
-            'html/ops',
-            'html/ops/ffmail',
-            'html/ops/mass_email',
-            'html/ops/remind_email',
-            'html/project',
-            'html/stats',
-            'html/user',
-            'html/user/img',
-            'html/user_profile',
-            'html/user_profile/images'
-        ])
+
+    directories = ('',
+                   'cgi-bin',
+                   'bin',
+                   'py',
+                   'py/Boinc',
+                   'templates',
+                   'upload',
+                   'download',
+                   'apps',
+                   'html',
+                   'html/cache',
+                   'html/inc',
+                   'html/inc/password_compat',
+                   'html/inc/random_compat',  
+                   'html/inc/ReCaptcha',
+                   'html/inc/ReCaptcha/RequestMethod',
+                   'html/languages',
+                   'html/languages/compiled',
+                   'html/languages/translations',
+                   'html/languages/project_specific_translations',
+                   'html/ops',
+                   'html/ops/ffmail',
+                   'html/ops/mass_email',
+                   'html/ops/remind_email',
+                   'html/project',
+                   'html/stats',
+                   'html/user',
+                   'html/user/img',
+                   'html/user_profile',
+                   'html/user_profile/images'
+    )
+    [ mkdir2(os.path.join(dest_dir, x)) for x in directories ]
 
     # For all directories that apache will put files in,
     # make them group-writeable and setGID.
@@ -290,82 +289,89 @@ def create_project_dirs(dest_dir):
     # any files or dirs created by apache will be owned by
     # our primary group (not Apache's).
     #
-    map(lambda d: os.chmod(dir(d), 02770),
-        [
-            'upload',
-            'html/cache',
-            'html/inc',
-            'html/languages',
-            'html/languages/compiled',
-            'html/user_profile/images'
-        ])
+    directories =  [
+        'upload',
+        'html/cache',
+        'html/inc',
+        'html/languages',
+        'html/languages/compiled',
+        'html/user_profile/images',
+    ]
+    for d in directories:
+        os.chmod(os.path.join(dest_dir, d), 0o2770)
 
 def install_boinc_files(dest_dir, install_web_files, install_server_files):
     """Copy files from source dir to project dir.
         Used by the upgrade script, so don't copy sample files to real name."""
 
-    def dir(*dirs):
-        return apply(os.path.join,(dest_dir,)+dirs)
+    def dest(*dirs):
+        location = dest_dir
+        for d in dirs:
+            location = os.path.join(location, d )
+        return location
 
     create_project_dirs(dest_dir);
 
     # copy html/ops files in all cases.
     # The critical one is db_update.php,
     # which is needed even for a server_only upgrade
-
-    install_glob(srcdir('html/ops/*.php'), dir('html/ops/'))
+    install_glob(srcdir('html/ops/*.php'), dest('html/ops/'))
 
     if install_web_files:
-        install_glob(srcdir('html/inc/*.inc'), dir('html/inc/'))
-        install_glob(srcdir('html/inc/*.php'), dir('html/inc/'))
-        install_glob(srcdir('html/inc/password_compat/*.inc'), dir('html/inc/password_compat/'))
-        install_glob(srcdir('html/inc/random_compat/*.inc'), dir('html/inc/random_compat/'))
-        install_glob(srcdir('html/inc/ReCaptcha/*.php'), dir('html/inc/ReCaptcha/'))
-        install_glob(srcdir('html/inc/ReCaptcha/RequestMethod/*.php'), dir('html/inc/ReCaptcha/RequestMethod'))
-        install_glob(srcdir('html/inc/*.dat'), dir('html/inc/'))
-        install_glob(srcdir('html/ops/*.css'), dir('html/ops/'))
-        install_glob(srcdir('html/ops/ffmail/sample*'), dir('html/ops/ffmail/'))
-        install_glob(srcdir('html/ops/mass_email/sample*'), dir('html/ops/mass_email/'))
-        install_glob(srcdir('html/ops/remind_email/sample*'), dir('html/ops/remind_email/'))
-        install_glob(srcdir('html/user/*.php'), dir('html/user/'))
-        install_glob(srcdir('html/user/*.inc'), dir('html/user/'))
-        install_glob(srcdir('html/user/*.css'), dir('html/user/'))
-        install_glob(srcdir('html/user/*.txt'), dir('html/user/'))
-        install_glob(srcdir('html/user/*.js'), dir('html/user/'))
-        install_glob(srcdir('html/user/*.png'), dir('html/user/img'))
-        install_glob(srcdir('html/user/*.gif'), dir('html/user/img'))
-        install_glob(srcdir('html/user/img/*.*'), dir('html/user/img'))
-        if not os.path.exists(dir('html/user/motd.php')):
-            shutil.copy(srcdir('html/user/sample_motd.php'), dir('html/user/motd.php'))
-        os.system("rm -f "+dir('html/languages/translations/*'))
-        install_glob(srcdir('html/languages/translations/*.po'), dir('html/languages/translations/'))
+        install_glob(srcdir('html/inc/*.inc'), dest('html/inc/'))
+        install_glob(srcdir('html/inc/*.php'), dest('html/inc/'))
+        install_glob(srcdir('html/inc/password_compat/*.inc'), dest('html/inc/password_compat/'))
+        install_glob(srcdir('html/inc/random_compat/*.inc'), dest('html/inc/random_compat/'))
+        install_glob(srcdir('html/inc/ReCaptcha/*.php'), dest('html/inc/ReCaptcha/'))
+        install_glob(srcdir('html/inc/ReCaptcha/RequestMethod/*.php'), dest('html/inc/ReCaptcha/RequestMethod'))
+        install_glob(srcdir('html/inc/*.dat'), dest('html/inc/'))
+        install_glob(srcdir('html/ops/*.css'), dest('html/ops/'))
+        install_glob(srcdir('html/ops/ffmail/sample*'), dest('html/ops/ffmail/'))
+        install_glob(srcdir('html/ops/mass_email/sample*'), dest('html/ops/mass_email/'))
+        install_glob(srcdir('html/ops/remind_email/sample*'), dest('html/ops/remind_email/'))
+        install_glob(srcdir('html/user/*.php'), dest('html/user/'))
+        install_glob(srcdir('html/user/*.inc'), dest('html/user/'))
+        install_glob(srcdir('html/user/*.css'), dest('html/user/'))
+        install_glob(srcdir('html/user/*.txt'), dest('html/user/'))
+        install_glob(srcdir('html/user/*.js'), dest('html/user/'))
+        install_glob(srcdir('html/user/*.png'), dest('html/user/img'))
+        install_glob(srcdir('html/user/*.gif'), dest('html/user/img'))
+        install_glob(srcdir('html/user/img/*.*'), dest('html/user/img'))
+        if not os.path.exists(dest('html/user/motd.php')):
+            shutil.copy(srcdir('html/user/sample_motd.php'), dest('html/user/motd.php'))
+        os.system("rm -f "+dest('html/languages/translations/*'))
+        install_glob(srcdir('html/languages/translations/*.po'), dest('html/languages/translations/'))
 
     # copy Python stuff
-    map(lambda (s): install(srcdir('sched',s), dir('bin',s)),
-        [ 'start' ])
-    force_symlink(dir('bin', 'start'), dir('bin', 'stop'))
-    force_symlink(dir('bin', 'start'), dir('bin', 'status'))
-    map(lambda (s): install(srcdir('py/Boinc',s), dir('py/Boinc',s)),
-        [
-            '__init__.py',
-            'add_util.py',
-            'boinc_db.py',
-            'boinc_project_path.py',
-            'boincxml.py',
-            'configxml.py',
-            'database.py',
-            'db_base.py',
-            'db_mid.py',
-            'projectxml.py',
-            'sched_messages.py',
-            'tools.py',
-            'util.py'
-        ])
-    print >>open(dir('bin', 'boinc_path_config.py'), 'w'), '''
+    install(srcdir('sched/start' ), dest('bin/start' ))    
+    force_symlink(dest('bin', 'start'), dest('bin', 'stop'))
+    force_symlink(dest('bin', 'start'), dest('bin', 'status'))
+    python_files = [
+        '__init__.py',
+        'add_util.py',
+        'boinc_db.py',
+        'boinc_project_path.py',
+        'boincxml.py',
+        'configxml.py',
+        'database.py',
+        'db_base.py',
+        'db_mid.py',
+        'projectxml.py',
+        'sched_messages.py',
+        'tools.py',
+        'util.py'
+    ]
+    for s in python_files:
+        install(srcdir("py/Boinc/" + s), dest('py/Boinc', s))
+
+    content = '''
 # Generated by make_project
-import sys, os
-sys.path.insert(0, os.path.join('%s', 'py'))
-''' % dest_dir
+import sys, os                                                                                                            
+sys.path.insert(0, os.path.join('{dest_dir}', 'py'))
+'''.format(dest_dir=dest_dir)
+    f = open(dest('bin', 'boinc_path_config.py'), "w")
+    f.write(content)
+    f.close()
 
     if not install_server_files:
         return
@@ -373,19 +379,17 @@ sys.path.insert(0, os.path.join('%s', 'py'))
     # copy backend (C++) programs;
     # rename current web daemons in case they're in use
 
-    if os.path.isfile(dir('cgi-bin', 'cgi')):
-        os.rename(dir('cgi-bin', 'cgi'), dir('cgi-bin', 'cgi.old'))
-    if os.path.isfile(dir('cgi-bin', 'fcgi')):
-        os.rename(dir('cgi-bin', 'fcgi'), dir('cgi-bin', 'fcgi.old'))
-        map(lambda (s): install(builddir('sched',s), dir('cgi-bin',s)),
-            [ 'fcgi'])
-    if os.path.isfile(dir('cgi-bin', 'file_upload_handler')):
-        os.rename(dir('cgi-bin', 'file_upload_handler'), dir('cgi-bin', 'file_upload_handler.old'))
-
-    map(lambda (s): install(builddir('sched',s), dir('cgi-bin',s)),
-        [ 'cgi', 'file_upload_handler'])
-    map(lambda (s): install(builddir('sched',s), dir('bin',s)),
-        [
+    if os.path.isfile(dest('cgi-bin', 'cgi')):
+        os.rename(dest('cgi-bin', 'cgi'), dest('cgi-bin', 'cgi.old'))
+    if os.path.isfile(dest('cgi-bin', 'fcgi')):
+        os.rename(dest('cgi-bin', 'fcgi'), dest('cgi-bin', 'fcgi.old'))
+        install(builddir('sched','fcgi'), dest('cgi-bin','fcgi'))
+    if os.path.isfile(dest('cgi-bin', 'file_upload_handler')):
+        os.rename(dest('cgi-bin', 'file_upload_handler'), dest('cgi-bin', 'file_upload_handler.old'))
+    cgi_script = [ 'cgi', 'file_upload_handler']
+    for f in cgi_script:
+        install(builddir('sched/' + f), dest('cgi-bin',f))
+    command = [
             'adjust_user_priority',
             'antique_file_deleter',
             'assimilator.py',
@@ -417,12 +421,14 @@ sys.path.insert(0, os.path.join('%s', 'py'))
             'trickle_deadline',
             'trickle_echo',
             'update_stats',
-            'wu_check'
-        ])
-    map(lambda (s): install(builddir('vda',s), dir('bin',s)),
-        [ 'vda', 'vdad' ])
-    map(lambda (s): install(srcdir('tools',s), dir('bin',s)),
-        [
+            'wu_check',
+        ]
+    for f in command:
+        install(builddir('sched/' + f), dest('bin',f))
+    command = [ 'vda', 'vdad' ]
+    for f in command:
+        install(builddir('vda/' + f), dest('bin',f))
+    command = [
             'appmgr',
             'boinc_submit',
             'cancel_jobs',
@@ -441,13 +447,12 @@ sys.path.insert(0, os.path.join('%s', 'py'))
             'stage_file',
             'update_versions',
             'watch_tcp',
-            'xadd'
-        ])
-    map(lambda (s): install(srcdir('lib',s), dir('bin',s)),
-        [ 'crypt_prog' ])
-    map(lambda (s): install(srcdir('sched',s), dir('',s)),
-        [ 'db_dump_spec.xml' ])
-
+            'xadd',
+        ]
+    for f in command:
+        install(builddir('tools/' + f), dest('bin',f))
+    install(srcdir('lib/crypt_prog'), dest('bin','crypt_prog'))
+    install(srcdir('sched/db_dump_spec.xml' ), dest('','db_dump_spec.xml' ))
 
 class Project:
     def __init__(self,
@@ -471,7 +476,7 @@ class Project:
 
         self.project_dir   = project_dir or os.path.join(options.projects_dir, self.short_name)
 
-        self.config = configxml.ConfigFile(self.dir('config.xml')).init_empty()
+        self.config = configxml.ConfigFile(self.dest('config.xml')).init_empty()
         config = self.config.config
 
         # this is where default project config is defined
@@ -488,7 +493,11 @@ class Project:
         config.max_wus_to_send = 50
         config.daily_result_quota = 500
         config.disable_account_creation = 0
+        config.disable_account_creation_rpc = 0
+        config.account_creation_rpc_require_consent = 0
         config.disable_web_account_creation = 0
+        config.enable_login_mustagree_termsofuse = 0
+        config.enable_privacy_by_default = 0
         config.show_results = 1
         config.cache_md5_info = 1
         config.sched_debug_level = 3
@@ -513,24 +522,27 @@ class Project:
             config.min_sendwork_interval = 6
         self.scheduler_url = os.path.join(cgi_url     , 'cgi')
 
-    def dir(self, *dirs):
-        return apply(os.path.join,(self.project_dir,)+dirs)
-
-    def keydir(self, *dirs):
-        return apply(os.path.join,(self.config.config.key_dir,)+dirs)
+    def dest(self, *dirs):
+        location = self.project_dir
+        for x in dirs:
+            location = os.path.join(location, x)
+        return location
+        
+    def keydir(self, location):
+            return os.path.join(self.config.config.key_dir, location)
 
     def logdir(self):
         return os.path.join(self.project_dir, "log_"+self.config.config.host)
 
     def create_keys(self):
-        if not os.path.exists(self.keydir()):
-            os.mkdir(self.keydir())
+        if not os.path.exists(self.config.config.key_dir):
+            os.mkdir(self.config.config.key_dir)
         _gen_key(self.keydir('upload'))
         _gen_key(self.keydir('code_sign'))
 
     def create_logdir(self):
         os.mkdir(self.logdir())
-        os.chmod(self.logdir(), 02770)
+        os.chmod(self.logdir(), 0o2770)
 
     def keys_exist(self):
         keys = ['upload_private', 'upload_public',
@@ -541,8 +553,8 @@ class Project:
 
     # create new project.  Called only from make_project
     def install_project(self):
-        if os.path.exists(self.dir()):
-            raise SystemExit('Project directory "%s" already exists; this would clobber it!'%self.dir())
+        if os.path.exists(self.dest()):
+            raise SystemExit('Project directory "%s" already exists; this would clobber it!'%self.dest())
 
         verbose_echo(1, "Creating directories");
 
@@ -559,32 +571,35 @@ class Project:
         # Create the project log directory
         self.create_logdir()
 
-        install_boinc_files(self.dir(), True, not self.web_only)
+        install_boinc_files(self.dest(), True, not self.web_only)
 
         # copy sample web files to final names
         install(srcdir('html/user/sample_index.php'),
-            self.dir('html/user/index.php'))
+            self.dest('html/user/index.php'))
         install(srcdir('html/user/sample_bootstrap.min.css'),
-            self.dir('html/user/bootstrap.min.css'))
+            self.dest('html/user/bootstrap.min.css'))
         install(srcdir('html/user/sample_bootstrap.min.js'),
-            self.dir('html/user/bootstrap.min.js'))
+            self.dest('html/user/bootstrap.min.js'))
         install(srcdir('html/user/sample_jquery.min.js'),
-            self.dir('html/user/jquery.min.js'))
+            self.dest('html/user/jquery.min.js'))
         install(srcdir('html/project.sample/project.inc'),
-            self.dir('html/project/project.inc'))
+            self.dest('html/project/project.inc'))
         install(srcdir('html/project.sample/project_specific_prefs.inc'),
-            self.dir('html/project/project_specific_prefs.inc'))
+            self.dest('html/project/project_specific_prefs.inc'))
         install(srcdir('html/project.sample/cache_parameters.inc'),
-            self.dir('html/project/cache_parameters.inc'))
-        install(srcdir('tools/project.xml'), self.dir('project.xml'))
-        install(srcdir('tools/gui_urls.xml'), self.dir('gui_urls.xml'))
+            self.dest('html/project/cache_parameters.inc'))
+        install(srcdir('tools/project.xml'), self.dest('project.xml'))
+        install(srcdir('tools/gui_urls.xml'), self.dest('gui_urls.xml'))
         if not self.production:
-            install(srcdir('test/uc_result'), self.dir('templates/uc_result'))
-            install(srcdir('test/uc_wu_nodelete'), self.dir('templates/uc_wu'))
+            install(srcdir('test/uc_result'), self.dest('templates/uc_result'))
+            install(srcdir('test/uc_wu_nodelete'), self.dest('templates/uc_wu'))
 
-        f = open(self.dir('html/user', 'schedulers.txt'), 'w')
-        print >>f, "<!-- <scheduler>" + self.scheduler_url.strip() + "</scheduler> -->"
-        print >>f, "<link rel=\"boinc_scheduler\" href=\"" + self.scheduler_url.strip()+ "\">"
+        content = '''
+<!-- <scheduler>{url}</scheduler> -->"
+<link rel=\"boinc_scheduler\" href=\"url"\">
+        '''.format(url=self.scheduler_url.strip())    
+        f = open(self.dest('html/user', 'schedulers.txt'), 'w')
+        f.write(content)
         f.close()
 
         if self.no_db:
@@ -598,31 +613,33 @@ class Project:
                 )
 
         verbose_echo(1, "Writing config files")
-
         self.config.write()
 
         # create symbolic links to the CGI and HTML directories
         verbose_echo(1, "Linking CGI programs")
         if options.__dict__.get('cgi_dir'):
-            force_symlink(self.dir('cgi-bin'), os.path.join(options.cgi_dir, self.short_name))
+            force_symlink(self.dest('cgi-bin'), os.path.join(options.cgi_dir, self.short_name))
         if options.__dict__.get('html_dir'):
-            force_symlink(self.dir('html/user'), os.path.join(options.html_dir, self.short_name))
-            force_symlink(self.dir('html/ops'), os.path.join(options.html_dir, self.short_name+'_admin'))
+            force_symlink(self.dest('html/user'), os.path.join(options.html_dir, self.short_name))
+            force_symlink(self.dest('html/ops'), os.path.join(options.html_dir, self.short_name+'_admin'))
 
     def http_password(self, user, password):
         'Adds http password protection to the html/ops directory'
-        passwd_file = self.dir('html/ops', '.htpassword')
-        f = open(self.dir('html/ops', '.htaccess'), 'w')
-        print >>f, "AuthName '%s Administration'" % self.long_name
-        print >>f, "AuthType Basic"
-        print >>f, "AuthUserFile %s" % passwd_file
-        print >>f, "require valid-user"
+        passwd_file = self.dest('html/ops', '.htpassword')
+        content = '''
+AuthName '{long_name} Administration'
+AuthType Basic
+AuthUserFile {passwd_file}
+require valid-user
+'''.format(long_name=self.long_name, passwd_file=passwd_file)
+        f = open(self.dest('html/ops', '.htaccess'), 'w')
+        f.write(content)
         f.close()
         shell_call("htpassword -bc %s %s %s" % (passwd_file, user, password))
 
     def _run_sched_prog(self, prog, args='', logfile=None):
         verbose_shell_call("cd %s && ./%s %s >> %s.log 2>&1" %
-                           (self.dir('bin'), prog, args, (logfile or prog)))
+                           (self.dest('bin'), prog, args, (logfile or prog)))
 
     def start_servers(self):
         self.started = True
@@ -651,9 +668,9 @@ class Project:
             each_app = True
         else:
             raise SystemExit("test script error: invalid progname '%s'"%progname)
-        cmdline = apply(build_command_line, [''], kwargs)
+        cmdline = build_command_line('', kwargs)
         if each_app:
-            return map(lambda av: '-app %s %s'%(av.app.name,cmdline), self.app_versions)
+            return [ '-app %s %s'%(av.app.name,cmdline) for av in self.app_versions ]
         else:
             return [cmdline]
 
@@ -669,12 +686,14 @@ class Project:
     #     self.config.write()
 
     def start_stripcharts(self):
-        map(lambda l: self.copy(os.path.join('stripchart', l), 'cgi-bin/'),
-            [ 'stripchart.cgi', 'stripchart', 'stripchart.cnf',
-              'looper', 'db_looper', 'datafiles', 'get_load', 'dir_size' ])
+        cgi_bin = [ 'stripchart.cgi', 'stripchart', 'stripchart.cnf',
+            'looper', 'db_looper', 'datafiles', 'get_load', 'dir_size' ]
+        for f in cgi_bin:
+            self.copy(os.path.join('stripchart', f), 'cgi-bin/')
+
         macro_substitute('BOINC_DB_NAME', self.db_name, srcdir('stripchart/samples/db_count'),
-                         self.dir('bin/db_count'))
-        make_executable(self.dir('bin/db_count'))
+                         self.dest('bin/db_count'))
+        make_executable(self.dest('bin/db_count'))
 
         self._run_sched_prog('looper'    , 'get_load 1'                            , 'get_load')
         self._run_sched_prog('db_looper' , '"result" 1'                            , 'count_results')
