@@ -139,7 +139,11 @@ void TraverseSnapshots(std::string& current_snapshot_id, std::vector<std::string
 
     // Check to see if we have any children
     //
+#ifdef _VIRTUALBOX60_
+    rc = pSnapshot->get_ChildrenCount(&lCount);
+#else
     rc = pSnapshot->GetChildrenCount(&lCount);
+#endif
     if (SUCCEEDED(rc) && lCount) {
         rc = pSnapshot->get_Children(&pSnapshots);
         if (SUCCEEDED(rc)) {
@@ -831,6 +835,10 @@ int VBOX_VM::create_vm() {
             CComBSTR(string(virtual_machine_slot_directory + "\\shared").c_str()),
             TRUE,
             TRUE
+#ifdef _VIRTUALBOX60_
+            ,
+            CComBSTR("/")
+#endif
         );
         if (CHECK_ERROR(rc)) goto CLEANUP;
     }
@@ -844,6 +852,10 @@ int VBOX_VM::create_vm() {
             CComBSTR(virtualbox_scratch_directory.c_str()),
             TRUE,
             TRUE
+#ifdef _VIRTUALBOX60_
+            ,
+            CComBSTR("/")
+#endif
         );
         if (CHECK_ERROR(rc)) goto CLEANUP;
     }
