@@ -1418,10 +1418,12 @@ void process_request(char* code_sign_key) {
                 if (diff < config.min_sendwork_interval) {
                     ok_to_send_work = false;
                     log_messages.printf(MSG_NORMAL,
-                        "Not sending work - last request too recent, please wait: %f\n", config.min_sendwork_interval
+                        "Not sending work. Last request too recent. Please wait %d seconds.\n",
+                        (int)(config.min_sendwork_interval - diff)
                     );
                     sprintf(buf,
-                        "Not sending work - last request too recent, please wait: %d sec", (int)config.min_sendwork_interval
+                        "Not sending work. Last request too recent. Please wait %d seconds.\n",
+                        (int)(config.min_sendwork_interval - diff)
                     );
                     g_reply->insert_message(buf, "low");
 
